@@ -4,7 +4,7 @@ General tools useful to create NeXus format files.
 
 __author__ = "Diamond Light Source - Scientific Software"
 __email__ = "scientificsoftware@diamond.ac.uk"
-__version__ = "0.5.6"
+__version__ = "0.5.7"
 __version_tuple__ = tuple(int(x) for x in __version__.split("."))
 
 import re
@@ -149,7 +149,7 @@ def get_iso_timestamp(ts: str) -> str:
     return ts_iso
 
 
-def units_of_length(q: Any, to_base: Optional[bool] = False) -> pint.Quantity:
+def units_of_length(q: Any, to_base: Optional[bool] = False):  # -> pint.Quantity:
     """
     Check that a quantity of length is compatible with NX_LENGTH, defaulting to m if dimensionless.
 
@@ -178,7 +178,7 @@ def units_of_length(q: Any, to_base: Optional[bool] = False) -> pint.Quantity:
         )
 
 
-def units_of_time(q: str) -> pint.Quantity:
+def units_of_time(q: str):  # -> pint.Quantity:
     """
     Check that a quantity of time is compatible with NX_TIME, defaulting to s if dimensionless.
     Convert to seconds if time is passed as a fraction of it.
